import React, { useCallback } from "react";
import ChatBox, { ChatFrame } from "react-chat-plugin";
import { FaHeadset, FaMoon, FaSun, FaVolumeMute, FaVolumeUp } from "react-icons/fa";
import { nanoid } from "nanoid";
import firebase from "firebase/app";
import ReactTooltip from 'react-tooltip';
import axios from "axios";
import "./HawkingChatbot.css";
import "./HawkingChatbot.module.css";
import { db } from "./firebase";
import HAWKING_LOGO from "./hawking-logo.jpg";
import speakText from './speakText';

const HawkingChatbot = ({
  token,
  showChat = false
}) => {
  // LOCAL
  // const REACT_BACKEND_URL = 'http://127.0.0.1:8000/api/';
  // const REACT_PLATFORM_URL = 'http://localhost:3000/';
  // // PRODUCTION
  const REACT_BACKEND_URL = 'https://hawking.pythonanywhere.com/api/';
  const REACT_PLATFORM_URL = 'https://hawking.ro';
  const bearerToken = 'passToken5252021'; //await auth?.currentUser?.getIdToken(true);

  const [sessionID] = React.useState(nanoid(30));
  const [timer, setTimer] = React.useState(-1);
  const [showTypingIndicator, setShowTypingIndicator] = React.useState(false);
  const [speak, setSpeak] = React.useState(true);
  const [showChatbox, setShowChatbox] = React.useState(showChat);
  const [liveChat, setLiveChat] = React.useState(false);
  const [darkMode, setDarkMode] = React.useState(false);
  const [helperQuestions, setHelperQuestions] = React.useState(false);
  const [messages, setMessages] = React.useState([{
    author: {
      username: "Hawking",
      id: 2,
      avatarUrl: HAWKING_LOGO
    },
    text: "Hawking has joined the conversation 👋",
    type: "text",
    timestamp: +new Date()
  }, {
    author: {
      username: "Hawking",
      id: 2,
      avatarUrl: HAWKING_LOGO
    },
    text: "What's up?",
    type: "text",
    timestamp: +new Date()
  }]); // canceling live chat session after tab close/refresh

  React.useEffect(() => {
    const cancelLiveSession = ev => {
      ev.preventDefault();
      db.collection("users").doc(token).collection("livechats").doc(sessionID).delete();
    };

    window.addEventListener("beforeunload", cancelLiveSession);
  }, [token, sessionID]);
  const doSpeakText = React.useCallback(message => {
    // say text out loud
    if (speak) speakText(message);
  }, [speak]); // receiving messages from backend

  React.useEffect(() => {
    const unsubscribe = db.collection("users").doc(token).collection("messages").where("session", "==", sessionID).orderBy("timestamp", "asc").onSnapshot(function (snapshot) {
      snapshot.docChanges().forEach(function (change) {
        if (change.type === "added") {
          const newMessage = change.doc.data(); // message from Hawking

          if (newMessage.answer) {
            doSpeakText(newMessage.answer);
            setMessages(messages => [...messages, {
              author: {
                username: "Hawking",
                id: 2,
                avatarUrl: HAWKING_LOGO
              },
              text: newMessage.answer,
              type: "text",
              timestamp: newMessage.timestamp.seconds * 1000
            }]);
          }
        }
      });
    });
    return unsubscribe;
  }, [sessionID, token, doSpeakText]); // show/hide bot

  const handleClickIcon = () => {
    // toggle showChatbox
    setShowChatbox(!showChatbox);
  }; // send message to firebase db


  const handleOnSendMessage = useCallback(async message => {
    // seting that we asked some question or chose from the helper ones
    setHelperQuestions(false); // reseting timer to 0

    setTimer(0);
    setMessages(messages => [...messages, {
      author: {
        username: "You",
        id: 1,
        avatarUrl: "https://image.flaticon.com/icons/svg/2446/2446032.svg"
      },
      text: message,
      type: "text",
      timestamp: +new Date()
    }]); // append new user message to ChatBox

    if (!liveChat) {
      // talk to Hawking bot
      setShowTypingIndicator(true); // get Hawking response from backend

      const formData = new FormData();
      formData.append("token", token);
      formData.append("msg", message);
      formData.append("session", sessionID);

      try {
        if (bearerToken) {
          const response = await axios.request({
            method: "POST",
            url: `${REACT_BACKEND_URL}get_message`,
            data: formData,
            headers: {
              Authorization: `Bearer ${bearerToken}`
            }
          }); // if we received some similar/suggestion messages

          if (response.data?.suggestions?.length) {
            // display suggestions 
            setMessages(messages => [...messages, {
              author: {
                username: 'Hawking',
                id: 2,
                avatarUrl: HAWKING_LOGO
              },
              text: /*#__PURE__*/React.createElement("div", null, "Also try these suggestions.", response.data.suggestions.map((q, i) => {
                return /*#__PURE__*/React.createElement("div", {
                  className: "chatboxChoiceButton",
                  key: i,
                  onClick: () => handleOnSendMessage(q.text)
                }, q.text);
              })),
              type: 'text',
              timestamp: +new Date()
            }]);
          }
        }
      } catch (error) {
        console.log(error);
      }

      setShowTypingIndicator(false);
    } else {
      // send message directly to frontend/platform rather than to the algorithm
      db.collection("users").doc(token).collection("messages").doc().set({
        timestamp: firebase.firestore.Timestamp.now(),
        session: sessionID,
        message: message,
        answer: ""
      });
    }
  }, [liveChat, sessionID, token]); // start timer

  React.useEffect(() => {
    if (timer !== -1) return;
    window.setInterval(function () {
      setTimer(timer => timer + 1);
    }, 1000);
  }, [timer]); // use timer

  React.useEffect(() => {
    // if user didnt interract with the chatbot for N seconds, engage with him/her
    // and if we haven't displayed any helper questions yet
    if (timer >= 5 && !helperQuestions && !liveChat) {
      // reset timer
      setTimer(0);
      setShowTypingIndicator(true);
      setHelperQuestions(true); // engage with user

      (async () => {
        const response = await axios.request({
          method: "GET",
          url: `${REACT_BACKEND_URL}get_engagement_questions?token=${token}`,
          headers: {
            Authorization: `Bearer ${bearerToken}`
          }
        });
        const engagementMessage = response.data.engagementMessage;
        const question1 = response.data.q1;
        const question2 = response.data.q2;
        setShowTypingIndicator(false); // display engagement questions 

        setMessages(messages => [...messages, {
          author: {
            username: 'Hawking',
            id: 2,
            avatarUrl: HAWKING_LOGO
          },
          text: /*#__PURE__*/React.createElement("div", null, engagementMessage, /*#__PURE__*/React.createElement("div", {
            className: "chatboxChoiceButton",
            onClick: () => handleOnSendMessage(question1)
          }, question1), /*#__PURE__*/React.createElement("div", {
            className: "chatboxChoiceButton",
            onClick: () => handleOnSendMessage(question2)
          }, question2)),
          type: 'text',
          timestamp: +new Date()
        }]);
      })();
    }
  }, [timer, token, handleOnSendMessage, helperQuestions, liveChat]); // talk to a live person from the platform

  const goLiveChat = () => {
    if (!liveChat) {
      // toggling live chat on in firestore
      db.collection("users").doc(token).collection("livechats").doc(sessionID).set({
        timestamp: firebase.firestore.Timestamp.now(),
        token: token,
        session: sessionID
      });
      setShowTypingIndicator(true); // append info message to frontend

      setMessages(messages => [...messages, {
        author: {
          username: "Hawking",
          id: 2,
          avatarUrl: HAWKING_LOGO
        },
        text: 'Live Chat is now active, if someone is available he/she will get in touch with you as soon as possible.',
        type: "text",
        timestamp: +new Date()
      }]);
      setShowTypingIndicator(false);
    } // setting live chat to on


    setLiveChat(true);
  }; // change theme


  const handleChangeTheme = () => {
    setDarkMode(!darkMode);
  };

  return /*#__PURE__*/React.createElement(ChatFrame, {
    chatbox: /*#__PURE__*/React.createElement("div", {
      className: darkMode ? 'chatbox-main-dark' : 'chatbox-main'
    }, /*#__PURE__*/React.createElement("div", {
      className: "chatBoxHeader"
    }, /*#__PURE__*/React.createElement("a", {
      href: REACT_PLATFORM_URL,
      target: "_blank",
      rel: "noreferrer",
      className: "chatBoxHeaderTitleHref"
    }, /*#__PURE__*/React.createElement("p", {
      className: "chatBoxHeaderTitle"
    }, "Hawking")), /*#__PURE__*/React.createElement("span", {
      className: "iconContainer"
    }, !liveChat && /*#__PURE__*/React.createElement("p", {
      className: "chatBoxHeaderItem",
      onClick: goLiveChat
    }, /*#__PURE__*/React.createElement(FaHeadset, {
      "data-tip": true,
      "data-for": "liveChatTip"
    }), /*#__PURE__*/React.createElement(ReactTooltip, {
      className: "myTooltip",
      id: "liveChatTip",
      place: "bottom",
      effect: "solid"
    }, "Connect to Live Chat")), /*#__PURE__*/React.createElement("p", {
      className: "chatBoxHeaderItem",
      onClick: handleChangeTheme
    }, !darkMode ? /*#__PURE__*/React.createElement(FaSun, {
      "data-tip": true,
      "data-for": "themeTip"
    }) : /*#__PURE__*/React.createElement(FaMoon, {
      "data-tip": true,
      "data-for": "themeTip"
    }), /*#__PURE__*/React.createElement(ReactTooltip, {
      className: "myTooltip",
      id: "themeTip",
      place: "bottom",
      effect: "solid"
    }, "Switch theme")))), /*#__PURE__*/React.createElement(ChatBox, {
      onSendMessage: handleOnSendMessage,
      userId: 1,
      placeholder: "Write message..",
      messages: messages,
      width: "500px",
      showTypingIndicator: showTypingIndicator,
      activeAuthor: {
        username: "Hawking",
        id: 2,
        avatarUrl: HAWKING_LOGO
      }
    }), /*#__PURE__*/React.createElement("div", {
      className: "hawkingIconContainer"
    }, /*#__PURE__*/React.createElement("img", {
      src: HAWKING_LOGO,
      className: "hawkingIcon hawkingIconActive",
      onClick: handleClickIcon,
      alt: "hawking"
    }))),
    showChatbox: showChatbox
  }, /*#__PURE__*/React.createElement("div", {
    className: "hawkingIconContainer"
  }, /*#__PURE__*/React.createElement("img", {
    src: HAWKING_LOGO,
    className: "hawkingIcon hawkingIconAnimation",
    onClick: handleClickIcon,
    alt: "hawking"
  })));
};

export default HawkingChatbot;